/** \file playertype_function.cpp
* \brief プレイヤータイプに関するバインド用ファイル
* \auther Shun.E
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <iostream>
#include <rcsc/action/basic_actions.h>
#include <rcsc/player/player_agent.h>
#include <rcsc/player/debug_client.h>
#include <rcsc/common/audio_memory.h>
#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>
#include "playertype_function.h"
#include "izk_main.h"

using namespace luabind;
using namespace rcsc;

/** \fn l_PlayerId
* \brief ID 番号を取得する.
* \return プレイヤーのID 番号
*/
int
l_PlayerId()
{
	const WorldModel & tok_wm = izk_agent->world();
	return tok_wm.self().playerType().id();
}

/** \fn l_PlayerSize
* \brief プレイヤータイプで設定されたプレイヤーの大きさを取得する.
* \return プレイヤーの大きさ
*/
double
l_PlayerSize()
{
	const WorldModel & tok_wm = izk_agent->world();
	return tok_wm.self().playerType().playerSize();
}

/** \fn l_StaminaIncMax()
* \brief スタミナ回復量最大値を取得する.
* \return スタミナ回復量最大値
*/
double
l_StaminaIncMax()
{
	const WorldModel & tok_wm = izk_agent->world();
	return tok_wm.self().playerType().staminaIncMax();
}

/** \fn l_DashPowerRate()
* \brief ダッシュ効率を取得する.
* \return ダッシュ効率
*/
double
l_DashPowerRate()
{
	const WorldModel & tok_wm = izk_agent->world();
	return tok_wm.self().playerType().dashPowerRate();
}

/** \fn l_PlayerDecay()
* \brief 速度減衰率を取得する.
* \return 速度減衰率
*/
double
l_PlayerDecay()
{
	const WorldModel & tok_wm = izk_agent->world();
	return tok_wm.self().playerType().playerDecay();
}

/** \fn l_PlayerSpeedMax()
* \brief プレイヤーの最大スピードを取得する.
* \return プレイヤーの最大スピード
*/
double
l_PlayerSpeedMax()
{
	const WorldModel & tok_wm = izk_agent->world();
	return tok_wm.self().playerType().playerSpeedMax();
}

/** \fn l_KickableMargin()
* \brief プレイヤーのキック可能距離を取得する.
* \return プレイヤーのキック可能距離
*/
double
l_KickableMargin()
{
	const WorldModel & tok_wm = izk_agent->world();
	return tok_wm.self().playerType().kickableMargin();
}

/** \fn l_DashPowerToKeepSpeed
* \brief 体力が維持可能なスピードにするためのダッシュパワーを取得する.
* \param speed : スピード, effort : 努力値
* \return ダッシュパワー
*/
double
l_DashPowerToKeepSpeed(double speed, double effort)
{
	const WorldModel & tok_wm = izk_agent->world();
	return tok_wm.self().playerType().getDashPowerToKeepSpeed(speed, effort);
}

/** \fn openFile
* \brief Lua ファイルを開く
*/
void
PlayerTypeFunction::openFile()
{
}

/** \fn bindFunc
* \brief 関数のバインドを行う
*/
void
PlayerTypeFunction::bindFunc()
{
	// 関数やクラスを登録
	module(Luas)
	[
		def("PlayerId", &l_PlayerId),
		def("PlayerSize", &l_PlayerSize),
		def("StaminaIncMax", &l_StaminaIncMax),
		def("DashPowerRate", &l_DashPowerRate),
		def("PlayerDecay", &l_PlayerDecay),
		def("PlayerSpeedMax", l_PlayerSpeedMax),
		def("KickableMargin", l_KickableMargin),
		def("DashPowerToKeepSpeed",l_DashPowerToKeepSpeed)
	];
}

