/** \file gamemode_function.cpp
*   \brief ゲームモードに関するバインド用ファイル
*   \auther Shun.E
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>

#include <rcsc/action/basic_actions.h>
#include <rcsc/player/player_agent.h>
#include <rcsc/player/debug_client.h>

#include <rcsc/common/audio_memory.h>
#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>


#include "bhv_set_play.h"
#include "izk_main.h"
#include "gamemode_function.h"

using namespace luabind;
using namespace rcsc;

/** \fn l_nowGameModeType
*   \brief ゲームモードタイプを取得
*   \return ゲームモードタイプを数値で返す
*/
int
l_nowGameModeType()
{
    const WorldModel & tok_wm = izk_agent->world();	// WorldModelの設定
    switch ( tok_wm.gameMode().type() ) {
       case GameMode::KickOff_: 	return  0; break;
       case GameMode::AfterGoal_: 	return  1; break;
       case GameMode::BackPass_: 	return  2; break;
       case GameMode::IndFreeKick_:	return  3; break;
       case GameMode::KickIn_:		return  4; break;
       case GameMode::GoalKick_:	return  5; break;
       case GameMode::CornerKick_: 	return  6; break;
       case GameMode::FoulCharge_:	return  7; break;
       case GameMode::FoulPush_:	return  8; break;
       case GameMode::PenaltySetup_:	return  9; break;
       case GameMode::PenaltyReady_:	return 10; break;
       case GameMode::PenaltyTaken_:	return 11; break;
       case GameMode::PenaltyScore_:    return 12; break;
       case GameMode::PenaltyMiss_:	return 13; break;
       case GameMode::PenaltyOnfield_:	return 14; break;
       case GameMode::PenaltyFoul_:	return 15; break;
       case GameMode::PlayOn:		return 16; break;
    }
    return -1;
}

/** \fn l_isPlayOn()
*   \brief プレイオンかどうかの判定をする.
*   \return true : プレイオン, false : プレイオンではない
*/
bool
l_isPlayOn()
{
    // プレイオンかどうか
    const WorldModel & tok_wm = izk_agent->world();	// WorldModelの設定
    return tok_wm.gameMode().type() == GameMode::PlayOn;
}

/** \fn l_isPenaltyKick()
*   \brief ペナルティキックかどうかの判定をする.
*   \return true : ペナルティキック, false : ペナルティキックではない
*/
bool
l_isPenaltyKick()
{
    // ペナルティキックかどうか
    const WorldModel & tok_wm = izk_agent->world(); // WorldModelの設定
    return tok_wm.gameMode().isPenaltyKickMode();
}


/** \fn l_nowGameModeSide
*   \brief ゲームモードサイドを取得
*   \return ゲームモードサイドを数値で返す
*/
int
l_nowGameModeSide()
{
    const WorldModel & tok_wm = izk_agent->world();	// WorldModelの設定
    switch ( tok_wm.gameMode().side() ) {
       case NEUTRAL: 	return 0; break;
       case LEFT: 	return 1; break;
       case RIGHT: 	return 2; break;
    }
    return -1;
}

/** \fn l_getourSide()
*   \brief サイドを取得
*   \return サイドを数値で返す
*/
int
l_getourSide()
{
   const WorldModel & tok_wm = izk_agent->world();	// WorldModelの設定
   if (tok_wm.ourSide() == NEUTRAL) {
	return 0;
   }

   if (tok_wm.ourSide() == LEFT) {
	return 1;
   }

   if (tok_wm.ourSide() == RIGHT) {
	return 2;
   }
   return -1;
}

/** \fn l_get_set_play_dash_power()
*   \brief セットプレイのダッシュパワーを返す
*   \return セットプレイのダッシュパワー
*/
double
l_get_set_play_dash_power()
{
   return Bhv_SetPlay::get_set_play_dash_power();
}

/** \fn openFile
*   \brief Luaファイルを開く
*/
void
GameModeFunction::openFile()
{
    int ret;

    // ゲームモードファイルを読み込む
    ret = luaL_loadfile(Luas, "luaLib/GameMode/GameMode.lua") 
			|| lua_pcall(Luas, 0,0,0);
    if(ret != 0){
      std::cerr << "Lua: lua_GameMode.lua読み込み失敗\n";
      std::cerr << "Error: " <<  lua_tostring(Luas, -1) << std::endl;;
      exit(-1); // ちゃんとエラー処理するべき．これは酷い．．
    }
}

/** \fn bindFunc
*   \brief 関数のバインドを行う
*/
void
GameModeFunction::bindFunc()
{
    // 関数やクラスを登録
    module(Luas)
    [
        def("isPlayOn", &l_isPlayOn),
        def("isPenaltyKick", &l_isPenaltyKick),

        def("nowGameModeType", &l_nowGameModeType),
        def("getourSide", &l_getourSide),
        def("nowGameModeSide", &l_nowGameModeSide),
        def("get_set_play_dash_power", &l_get_set_play_dash_power)
    ];
}
